------------------------------------------------------------------------------
--                                                                          --
--                            Matreshka Project                             --
--                                                                          --
--         Localization, Internationalization, Globalization for Ada        --
--                                                                          --
--                        Runtime Library Component                         --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2017, Vadim Godunko <vgodunko@gmail.com>                     --
-- All rights reserved.                                                     --
--                                                                          --
-- Redistribution and use in source and binary forms, with or without       --
-- modification, are permitted provided that the following conditions       --
-- are met:                                                                 --
--                                                                          --
--  * Redistributions of source code must retain the above copyright        --
--    notice, this list of conditions and the following disclaimer.         --
--                                                                          --
--  * Redistributions in binary form must reproduce the above copyright     --
--    notice, this list of conditions and the following disclaimer in the   --
--    documentation and/or other materials provided with the distribution.  --
--                                                                          --
--  * Neither the name of the Vadim Godunko, IE nor the names of its        --
--    contributors may be used to endorse or promote products derived from  --
--    this software without specific prior written permission.              --
--                                                                          --
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS      --
-- "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT        --
-- LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR    --
-- A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT     --
-- HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,   --
-- SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED --
-- TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   --
-- PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF   --
-- LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING     --
-- NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS       --
-- SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.             --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 5813 $ $Date: 2017-12-09 18:22:32 +0200 (Суб, 09 дек 2017) $
------------------------------------------------------------------------------

generic
   type Element_Type is private;

   with procedure Component
    (Self    : aliased Element_Type;
     Name    : League.Strings.Universal_String;
     Value   : out League.Holders.Holder;
     Success : out Boolean);

package League.Holders.Generic_Compound_Holders is

   pragma Preelaborate;

   Value_Tag : constant Tag;

   function Element (Self : Holder) return Element_Type;
   --  Returns internal value.

   procedure Replace_Element (Self : in out Holder; To : Element_Type);
   --  Set value. Tag of the value must be set before this call.

   function To_Holder (Item : Element_Type) return Holder;
   --  Creates new Value from specified value.

private

   type Element_Container is new Abstract_Container with record
      Value : aliased Element_Type;
   end record;

   overriding function Constructor
    (Is_Empty : not null access Boolean) return Element_Container;

   overriding procedure Component
    (Self    : not null access Element_Container;
     Name    : League.Strings.Universal_String;
     Value   : out Container_Access;
     Success : out Boolean);

   Value_Tag : constant Tag := Tag (Element_Container'Tag);

end League.Holders.Generic_Compound_Holders;
